///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to "CELayoutEditor.cpp".
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////

// Standard wxWidget includes
#include "wx/wx.h"
// Utility class for convertion between CEGUI- and wxWidgets strings
#include "StringHelper.h"
#include "wx/docview.h"
#include "EditorView.h"
#include "CEImagesetEditor.h"
#include "EditorFrame.h"
#include "EditorGLCanvas.h"
#include "PropertiesPanel.h"

IMPLEMENT_DYNAMIC_CLASS(EditorView, wxView)

//-----------------------------------------------------------------------
EditorView::EditorView()
{ 
  frame = (EditorFrame*)NULL;
}

//-----------------------------------------------------------------------
bool EditorView::OnCreate(wxDocument *document, long WXUNUSED(flags) )
{
  // Single-window mode
  frame = GetMainFrame();
  m_glcanvasImageset = frame->getCanvas();
  m_glcanvasImageset->setView (this);
  m_glcanvasImageset->Reset();
  m_glcanvasImageset->Enable();
  m_PropsPanel = frame->getPropsPanel();
  m_PropsPanel->Enable();
  m_PropsPanel->setView (this);

  // Associate the appropriate frame with this view.
  SetFrame(frame);

  // Tell the frame about the document
  frame->SetDocument((EditorDocument*)document);


//  updateFrameTitle();

  // Make sure the document manager knows that this is the
  // current view.
  Activate(true);
        
  return true;
}

//-----------------------------------------------------------------------
bool EditorView::OnClose(bool WXUNUSED(deleteWindow))
{
    if (!GetDocument()->Close())
    {
        return false;
    }
  
    m_glcanvasImageset->ClearBackground();
    m_glcanvasImageset->setView ((EditorView*) NULL);
    m_glcanvasImageset->Disable();
    m_glcanvasImageset = (EditorGLCanvas*) NULL;  
    
    m_PropsPanel->Disable();

    SetFrame((wxFrame*) NULL);

    // Tell the frame about the document
    frame->SetDocument((EditorDocument*) NULL);
    frame = (EditorFrame*) NULL;

    Activate(false);
    
    //updateFrameTitle();
    
    return true;
}

//-----------------------------------------------------------------------
void EditorView::OnUpdate(wxView* WXUNUSED(sender), wxObject* WXUNUSED(hint))
{
  if (frame && frame->IsShown())
  {
		//wxLogDebug(wxT("Needs repaint."));
    m_glcanvasImageset->Refresh(false);
		m_glcanvasImageset->Render();
  }
}

//-----------------------------------------------------------------------
void EditorView::OnChangeFilename()
{
  updateTitle();
}


// possible actions on the document 
void EditorView::OnChangeImageFile(const wxString& newfilename)
{
  m_PropsPanel->setImageFileName(newfilename);
  m_glcanvasImageset->setImage(newfilename);
}

void EditorView::OnChangeImagesetName(const wxString& newname)
{
	m_PropsPanel->setImagesetName(newname);
}

void EditorView::OnChangeNativeRes(const wxPoint& newnativeres)
{
	m_PropsPanel->setNativeRes(newnativeres);
}

void EditorView::OnChangeAutoScaled(const bool newAutoScaled)
{
	m_PropsPanel->setAutoScaled(newAutoScaled);
}

void EditorView::OnAddRegion(const wxString &name, const wxRect &dim)
{ 
  m_PropsPanel->addRegion(name, dim);
  m_glcanvasImageset->addRegion(name, dim);
}

void EditorView::OnDelRegion(const wxString &name)
{
  m_PropsPanel->delRegion(name);
  m_glcanvasImageset->delRegion(name);
}

void EditorView::OnDelAllRegion()
{
  m_PropsPanel->delAllRegion();
  m_glcanvasImageset->delAllRegion();
}

void EditorView::OnMoveScaleRegion(const wxString &name, const wxRect &dim)
{
  m_PropsPanel->movescaleRegion(name, dim);
  m_glcanvasImageset->movescaleRegion(name, dim);
}

void EditorView::OnRenameRegion(const wxString &currentname, const wxString &newname)
{
	//m_PropsPanel->addRegion(name, dim);
}

// Update frame section
//-----------------------------------------------------------------------
/** Updates the mainframe's title bar according to the currently open document.*/
void EditorView::updateTitle()
{
  wxString title (wxTheApp->GetAppName());
  
  if (GetDocument())
  {
    title << wxT("- [") << GetDocument()->GetFilename() << wxT("]");
  }
  if (frame)
  {
    frame->SetTitle(title);
  }
}

/** Updates the Mouse part of the status bar.*/
void EditorView::updateMouseStatusBar(const int posx, const int posy)
{

	wxChar statusInfo [100];
	wxSnprintf (statusInfo, sizeof(statusInfo), wxT("pos[%d, %d]"), posx, posy) ;

  if (frame)
  {
  frame->GetStatusBar()->SetStatusText(statusInfo, 0);
  }
}
  
/** Updates the View part of the status bar.*/
void EditorView::updateDocStatusBar(const float zoom, const int width, const int height)
{
  
	wxChar statusInfo [100];
  
  int newzoom = zoom * 100;
	wxSnprintf (statusInfo, sizeof(statusInfo), wxT("size[%d, %d] zoom[%d]"), width, height, newzoom) ;
  
  if (frame)
  {
    frame->GetStatusBar()->SetStatusText(statusInfo, 1);
  }
}

//-----------------------------------------------------------------------
void EditorView::OnDraw(wxDC *dc)
{
}
