/************************************************************************
filename: 	CEGUIImagesetHandler.cpp
created:	21/2/2004
author:		Paul D Turner

purpose:	Implements the Imageset class
*************************************************************************/
/*************************************************************************
Crazy Eddie's GUI System (http://www.cegui.org.uk)
Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/
#include "ImagesetHandler.h"

#include "CEGUIExceptions.h"
#include "CEGUISystem.h"
#include "CEGUILogger.h"
#include "CEGUIXMLAttributes.h"


#include "EditorDocument.h"
#include "StringHelper.h"

// Start of CEGUI namespace section
using namespace CEGUI;


/*************************************************************************
Definition of constant data for Imageset (and sub-classes)
*************************************************************************/
// Declared in Imageset::xmlHandler
const CEGUI::String ImagesetHandler::ImagesetElement( (utf8*)"Imageset" );
const CEGUI::String ImagesetHandler::ImageElement( (utf8*)"Image" );
const char	ImagesetHandler::ImagesetImageFileAttribute[]		= "Imagefile";
const char	ImagesetHandler::ImagesetResourceGroupAttribute[]   = "ResourceGroup";
const char	ImagesetHandler::ImagesetNameAttribute[]			= "Name";
const char	ImagesetHandler::ImagesetNativeHorzResAttribute[]	= "NativeHorzRes";
const char	ImagesetHandler::ImagesetNativeVertResAttribute[]	= "NativeVertRes";
const char	ImagesetHandler::ImagesetAutoScaledAttribute[]		= "AutoScaled";
const char	ImagesetHandler::ImageNameAttribute[]				= "Name";
const char	ImagesetHandler::ImageXPosAttribute[]				= "XPos";
const char	ImagesetHandler::ImageYPosAttribute[]				= "YPos";
const char	ImagesetHandler::ImageWidthAttribute[]				= "Width";
const char	ImagesetHandler::ImageHeightAttribute[]				= "Height";
const char	ImagesetHandler::ImageXOffsetAttribute[]			= "XOffset";
const char	ImagesetHandler::ImageYOffsetAttribute[]			= "YOffset";

/*************************************************************************
SAX2 Handler methods
*************************************************************************/
void ImagesetHandler::elementStart(const String& element, const XMLAttributes& attributes)
{
	// handle an Image element (extract all element attributes and use data to define an Image for the Imageset)
	if (element == ImageElement)
    {
        wxString	name;

        if (attributes.exists (ImageNameAttribute))
            name = StringHelper::ToWXString(attributes.getValueAsString(ImageNameAttribute));

        wxRect	rect;

        if (attributes.exists (ImageXPosAttribute))
            rect.SetX(	(float)attributes.getValueAsInteger(ImageXPosAttribute) );
        if (attributes.exists (ImageYPosAttribute))
            rect.SetY( (float)attributes.getValueAsInteger(ImageYPosAttribute) );
        if (attributes.exists (ImageWidthAttribute))
            rect.SetWidth( (float)attributes.getValueAsInteger(ImageWidthAttribute) );
        if (attributes.exists (ImageHeightAttribute))
            rect.SetHeight( (float)attributes.getValueAsInteger(ImageHeightAttribute) );
        
        wxPoint	offset;

        if (attributes.exists (ImageXOffsetAttribute))
            offset.x	= (float)attributes.getValueAsInteger(ImageXOffsetAttribute, 0);
        if (attributes.exists (ImageYOffsetAttribute))
            offset.y	= (float)attributes.getValueAsInteger(ImageYOffsetAttribute, 0);
        

		//d_imageset->defineImage(name, rect, offset);
        m_document->HandleAddRegion(name, rect);
	}
	// handle root Imageset element
	else if (element == ImagesetElement)
	{
        if (attributes.exists (ImagesetNameAttribute))
        {
            wxString	name(StringHelper::ToWXString(attributes.getValueAsString(ImagesetNameAttribute)) );
            m_document->HandleChangeImagesetName(name);
        }

        if (attributes.exists (ImagesetImageFileAttribute))
        {
            wxString	filename(StringHelper::ToWXString(attributes.getValueAsString(ImagesetImageFileAttribute)) );
            m_document->HandleChangeImageFile(filename);
        }
        else 
        {
            wxString	filename(StringHelper::ToWXString(attributes.getValueAsString("FileName")) );
            m_document->HandleChangeImageFile(filename);
        }

		//Logger::getSingleton().logEvent("Started creation of Imageset '" + d_imageset->d_name + "' via XML file.", Informative);

		//
		// load auto-scaling configuration
		//
		int hres= 640, vres=480;

        if (attributes.exists (ImagesetNativeHorzResAttribute))
        {
		        // get native horizontal resolution
            hres = attributes.getValueAsInteger(ImagesetNativeHorzResAttribute, 640);
        } 
        if (attributes.exists (ImagesetNativeVertResAttribute))
        {
		    // get native vertical resolution
            vres = attributes.getValueAsInteger(ImagesetNativeVertResAttribute, 480);
        }
        m_document->HandleChangeNativeRes(wxPoint(hres, vres));
		//d_imageset->setNativeResolution(Size(hres, vres));


        bool autoscal = false;
        if (attributes.exists (ImagesetAutoScaledAttribute))
        {
		    // enable / disable auto-scaling for this Imageset according to the setting
            autoscal = attributes.getValueAsBool(ImagesetAutoScaledAttribute, false);
        }
        m_document->HandleChangeAutoScaled(autoscal);

    }
}

void ImagesetHandler::elementEnd(const String& element)
{
	if (element == ImagesetElement)
	{
		//Logger::getSingleton().logEvent("Finished creation of Imageset '" + d_imageset->d_name + "' via XML file.", Informative);
    m_document->Modify(true);
	}
}

