/////////////////////////////////////////////////////////////////////////////
// Name:        ImagesetEditor.cpp
// Purpose:     Demos PNG reading
// Author:      Julian Smart
// Modified by:
// Created:     04/01/98
// RCS-ID:      $Id: pngdemo.cpp,v 1.24 2005/08/24 17:47:07 ABX Exp $
// Copyright:   (c) Julian Smart
// Licence:     wxWindows license
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
#pragma implementation "ImagesetEditor.h"
#endif

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include "wx/image.h"

#include "PropertiesPanel.h"
#include "EditorGLCanvas.h"
#include "EditorView.h"
// Utility class for convertion between CEGUI- and wxWidgets strings
#include "StringHelper.h"

// Toolbar icons
#include "bitmaps/new.xpm"
#include "bitmaps/open.xpm"
#include "bitmaps/save.xpm"
#include "bitmaps/cut.xpm"
#include "bitmaps/copy.xpm"
#include "bitmaps/paste.xpm"
#include "bitmaps/delete.xpm"
#include "bitmaps/help.xpm"

BEGIN_EVENT_TABLE(PropertiesPanel, wxPanel)
	EVT_TEXT_ENTER(ID_TEXTCTRL_NAME, PropertiesPanel::OnChangeImagesetName)
  EVT_BUTTON(ISE_BUTTON_LOAD_IMAGE, PropertiesPanel::OnChangeImageFile)
	EVT_TEXT(ID_COMBO, PropertiesPanel::OnChangeNativeRes)
	EVT_CHECKBOX(ID_CHECKBOX, PropertiesPanel::OnChangeAutoScaled)
  EVT_TOOL(ID_GRID_ADD_ROW, PropertiesPanel::OnGridAddRow)
  EVT_TOOL(ID_GRID_DEL_ROW, PropertiesPanel::OnGridDelRow)
  EVT_GRID_SELECT_CELL(PropertiesPanel::OnGridCellSelect)
  EVT_GRID_EDITOR_SHOWN(PropertiesPanel::OnGridCellEdit)
  EVT_GRID_CELL_CHANGE(PropertiesPanel::OnGridCellChange) 
END_EVENT_TABLE()

const char PropertiesPanel::charParseNativeRes('*');
const wxString PropertiesPanel::EmmitterPropertiesPanel( "EmmitterPropertiesPanel" );

// Define a constructor for my canvas
PropertiesPanel::PropertiesPanel(wxWindow *parent):
wxPanel(parent, wxID_ANY),
mUniqueGuid(0)
{
  // TODO : init from a config file
  m_bImageTrackListSel = true;
}

// Define a constructor for my canvas
void PropertiesPanel::createContent()
{

    // to manage the layout of the props panel
    wxBoxSizer *sizerProps = new wxBoxSizer( wxVERTICAL );
    SetSizer( sizerProps );

    // general properties section
    wxStaticText *lblProps = new wxStaticText( this, ID_TEXT, wxT("Properties"), wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE );
    lblProps->SetBackgroundColour( *wxGREEN );
    sizerProps->Add( lblProps, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 0 );

    // START ----------------------------------------------------------------------------
    // imageset name selector 
    wxStaticText *lblImagesetName = new wxStaticText( this, ID_TEXT, wxT("imageset name"), wxDefaultPosition, wxDefaultSize, 0 );
    sizerProps->Add( lblImagesetName, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT|wxTOP, 5 );

    m_txtImagesetName = new wxTextCtrl( this, ID_TEXTCTRL_NAME, wxT(""), wxDefaultPosition, wxDefaultSize, 0 );

    sizerProps->Add( m_txtImagesetName, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 5 );
    // END ------------------------------------------------------------------------------

    // START ----------------------------------------------------------------------------
    // image file selector 
    wxStaticText *lblImageFile = new wxStaticText( this, ID_TEXT, wxT("image file"), wxDefaultPosition, wxDefaultSize, 0 );
    sizerProps->Add( lblImageFile, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT|wxTOP, 5 );

    wxBoxSizer *sizerImageFile = new wxBoxSizer( wxHORIZONTAL );

    m_txtImageFileName = new wxTextCtrl( this, ISE_TEXTCTRL_IMAGE, wxT(""), wxDefaultPosition, wxDefaultSize, 0 );
    sizerImageFile->Add( m_txtImageFileName, 1, wxGROW|wxALIGN_CENTER_HORIZONTAL|wxALL, 0 );

    m_btnLoadImage = new wxButton( this, ISE_BUTTON_LOAD_IMAGE, wxT("..."), wxDefaultPosition, wxSize(20,-1), 0 );
    sizerImageFile->Add( m_btnLoadImage, 0, wxALIGN_BOTTOM|wxALIGN_CENTER_HORIZONTAL|wxALL, 0 );

    // add the image file selector to the props panel
    sizerProps->Add( sizerImageFile, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 5 );
    // END ------------------------------------------------------------------------------

    // START ----------------------------------------------------------------------------
    // native resolution selector
    wxGridSizer *sizerNativeRes = new wxGridSizer( 2, 2, 0, 5 );

    wxStaticText *lblNativeRes = new wxStaticText( this, ID_TEXT, wxT("native resolution"),  wxDefaultPosition, wxDefaultSize, 0 );
    sizerNativeRes->Add( lblNativeRes, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT|wxTOP, 0 );

    sizerNativeRes->AddSpacer(0);

    wxString strs14[] = 
    {
        wxT("640*480"), 
        wxT("800*600"), 
        wxT("1024*768"), 
        wxT("1280*1024"), 
        wxT("1600*1200")
    };
    m_cbboxNativeRes = new wxComboBox( this, ID_COMBO, wxT(""), wxDefaultPosition, wxDefaultSize, 5, strs14, wxCB_DROPDOWN );
    sizerNativeRes->Add( m_cbboxNativeRes, 1, wxGROW|wxALIGN_CENTER|wxLEFT|wxRIGHT, 0 );

    // to choice if the image set is auto scaled
    m_ckboxAutoScaled = new wxCheckBox( this, ID_CHECKBOX, wxT("Auto scaled"), wxDefaultPosition, wxDefaultSize, 0 );
    sizerNativeRes->Add( m_ckboxAutoScaled, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0 );

    // add the native resolution selector to the props panel
    sizerProps->Add( sizerNativeRes, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5 );
    // END ------------------------------------------------------------------------------

    // sets properties section
    wxStaticText *lblImageSet = new wxStaticText( this, ID_TEXT, wxT("Image sets"), wxDefaultPosition, wxSize(250,-1), wxALIGN_CENTRE );
    lblImageSet->SetBackgroundColour( *wxGREEN );
    sizerProps->Add( lblImageSet, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 0 );

    // Create toolbar with icons-only
    m_toolbarPropsGrid = new wxToolBar(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTB_HORIZONTAL); // |wxNO_BORDER
    //
    m_toolbarPropsGrid->AddTool(ID_GRID_ADD_ROW, wxBitmap (new_xpm));
    m_toolbarPropsGrid->AddTool(ID_GRID_DEL_ROW, wxBitmap (cut_xpm));
    sizerProps->Add( m_toolbarPropsGrid, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT|wxTOP, 5 );
    // Don't forget this one!
    m_toolbarPropsGrid->Realize();

    // sets properties grid
    createPropsGrid(this);

    // add sets properties grid to the props panel
    sizerProps->Add( m_gridRegionProps, 1, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5 );
    // END ----------------------------------------------------------------------------
}

// Define a constructor for my canvas
void PropertiesPanel::reset()
{
    setImagesetName("");
    setImageFileName("");
    delAllRegion();

}


void PropertiesPanel::setImagesetName(const wxString &name)
{
    m_txtImagesetName->SetValue(name);
}

wxString PropertiesPanel::getImagesetName()
{
    return m_txtImagesetName->GetValue();
}

void PropertiesPanel::setImageFileName(const wxString &name)
{
    m_txtImageFileName->SetValue(name);
    if(name.IsEmpty())
    {
      m_gridRegionProps->Disable();
      m_toolbarPropsGrid->Disable();
    }
    else
    {
      m_gridRegionProps->Enable();
      m_toolbarPropsGrid->Enable();
    }
}

wxString PropertiesPanel::getImageFileName()
{
    return m_txtImageFileName->GetValue();
}

void PropertiesPanel::setNativeRes(const wxPoint &NativeRes)
{
    wxChar statusInfo [100];
    wxSnprintf (statusInfo, sizeof(statusInfo), wxT("%d%c%d"), NativeRes.x, charParseNativeRes, NativeRes.y) ;
    m_cbboxNativeRes->SetValue(statusInfo);
}

wxPoint PropertiesPanel::getNativeRes()
{
    double xNativeRes, yNativeRes;
    wxString xNative, yNative;
    wxString values = m_cbboxNativeRes->GetValue();
    xNative = values.BeforeFirst(charParseNativeRes);
    yNative = values.AfterFirst(charParseNativeRes);
    xNative.ToDouble(&xNativeRes);
    yNative.ToDouble(&yNativeRes);
    return wxPoint(xNativeRes, yNativeRes);
}

bool PropertiesPanel::getAutoScaled()
{
    return m_ckboxAutoScaled->GetValue();
}

void PropertiesPanel::setAutoScaled(const bool AutoScaled)
{
    m_ckboxAutoScaled->SetValue(AutoScaled);
}

int PropertiesPanel::getRegionIndex(const wxString &name)
{
    int rowsize = m_gridRegionProps->GetRows();
    for (int i = 0; i < rowsize; i++)
    {
        const wxString rowname = m_gridRegionProps->GetCellValue(i, 0);
        if(rowname.Cmp(name) == 0)
        {
            return i;
        }
    }
    // return -1 if not found
    return -1;
}

int PropertiesPanel::selectRegion(const wxString &name)
{
    int rowsindex = getRegionIndex(name);
    if(rowsindex >= 0)
    {
        m_gridRegionProps->SelectRow(rowsindex);
        // can be made an option of the grid (track selection)
        m_gridRegionProps->MakeCellVisible(rowsindex, 0);
        // to terminate
        return rowsindex;
    }
// return -1 if nothing selected
    return -1;
}


void PropertiesPanel::addRegion(const wxString &name, const wxRect &dim, int rowindex )
{
    if (rowindex == -1)
    {
        if(m_gridRegionProps->IsSelection())
        {
            wxArrayInt sel = m_gridRegionProps->GetSelectedRows();
            if (sel.Count() > 0)
            {
                rowindex = m_gridRegionProps->GetSelectedRows().Item(0);
            }
            else
            {
                rowindex = m_gridRegionProps->GetNumberRows();
            }
        }
        else
        {
            rowindex = m_gridRegionProps->GetNumberRows();
        }
    }

    m_gridRegionProps->InsertRows(rowindex, 1);

    m_gridRegionProps->SetCellAlignment(rowindex, 0, wxALIGN_LEFT, wxALIGN_CENTRE);
    m_gridRegionProps->SetCellValue(rowindex, 0, name);


    wxChar value [100];
    wxSnprintf (value, sizeof(value), wxT("%d"), dim.GetX());
    m_gridRegionProps->SetCellValue(rowindex, 1, value);

    wxSnprintf (value, sizeof(value), wxT("%d"), dim.GetY());
    m_gridRegionProps->SetCellValue(rowindex, 2, value);

    wxSnprintf (value, sizeof(value), wxT("%d"), dim.GetWidth());
    m_gridRegionProps->SetCellValue(rowindex, 3, value);

    wxSnprintf (value, sizeof(value), wxT("%d"), dim.GetHeight());
    m_gridRegionProps->SetCellValue(rowindex, 4, value);

    m_gridRegionProps->AutoSizeColumn(0, false);

    m_gridRegionProps->ForceRefresh();
}

void PropertiesPanel::movescaleRegion(const wxString &name, const wxRect &dim)
{ 
    int row = getRegionIndex(name);
    if (row != -1)
    {
        wxChar value [100];

        if(dim.GetX() != m_gridRegionProps->GetCellValue(row, 1))
        {
            wxSnprintf (value, sizeof(value), wxT("%d"), dim.GetX());
            m_gridRegionProps->SetCellValue(row, 1, value);
        }

        if(dim.GetY() != m_gridRegionProps->GetCellValue(row, 2))
        {
            wxSnprintf (value, sizeof(value), wxT("%d"), dim.GetY());
            m_gridRegionProps->SetCellValue(row, 2, value);
        }

        if(dim.GetWidth() != m_gridRegionProps->GetCellValue(row, 3))
        {
            wxSnprintf (value, sizeof(value), wxT("%d"), dim.GetWidth());
            m_gridRegionProps->SetCellValue(row, 3, value);
        }

        if(dim.GetHeight() != m_gridRegionProps->GetCellValue(row, 4))
        {
            wxSnprintf (value, sizeof(value), wxT("%d"), dim.GetHeight());
            m_gridRegionProps->SetCellValue(row, 4, value);
        } 
    }
}

wxRect PropertiesPanel::getRegion(const wxString &name)
{
    wxRect dim;
    return dim;
}

int PropertiesPanel::delRegion(const wxString &name)
{
    int i;
    int rowindex = -1;
    int rowsize = m_gridRegionProps->GetRows();
    wxString rowname;
    for (i = 0; i < rowsize; i++)
    {
        rowname = m_gridRegionProps->GetCellValue(i, 0);
        if(rowname.Cmp(name) == 0)
        {
            m_gridRegionProps->DeleteRows(i, 1);

            rowindex = i;
            // to terminate
            i = rowsize;
        }
    }
    // return -1 if nothing deleted
    return rowindex;

}

void PropertiesPanel::delAllRegion()
{ 
    if (m_gridRegionProps->GetNumberRows () > 0)
    {
        m_gridRegionProps->ClearGrid();

        m_gridRegionProps->DeleteRows(0, m_gridRegionProps->GetNumberRows());

    }
}


void PropertiesPanel::createPropsGrid(wxWindow *parent)
{
    m_gridRegionProps = new wxGrid(parent, ID_GRID, wxDefaultPosition, wxSize(190,160),wxWANTS_CHARS | wxSUNKEN_BORDER);

    m_gridRegionProps->CreateGrid( 0, 5, wxGrid::wxGridSelectCells );
    m_gridRegionProps->EnableDragRowSize(false);


    //SetColLabelSize(20);
    m_gridRegionProps->SetColLabelSize(20);
    m_gridRegionProps->SetRowLabelSize(20);
    m_gridRegionProps->SetRowLabelAlignment(wxALIGN_RIGHT, wxALIGN_CENTRE);

    //hack, a enlever
    m_gridRegionProps->SetColLabelValue(0, "Name");
    m_gridRegionProps->SetColSize(0, -1);

    m_gridRegionProps->SetColLabelValue(1, "x");
    m_gridRegionProps->SetColSize(1, 30);

    m_gridRegionProps->SetColLabelValue(2, "y");
    m_gridRegionProps->SetColSize(2, 30);

    m_gridRegionProps->SetColLabelValue(3, "w");
    m_gridRegionProps->SetColSize(3, 30);

    m_gridRegionProps->SetColLabelValue(4, "h");
    m_gridRegionProps->SetColSize(4, 30);

    m_gridRegionProps->SetDefaultCellAlignment(wxALIGN_RIGHT, wxALIGN_CENTRE);
    m_gridRegionProps->ForceRefresh();

}

void PropertiesPanel::changeImageTrackListSel(bool isActive)
{  
  m_bImageTrackListSel = isActive;
}

void PropertiesPanel::OnChangeImagesetName(wxCommandEvent& WXUNUSED(event))
{
	// Check this, because we might get this event during document closing
  if (view && view->GetDocument())
  {
	  EditorDocument* document = (EditorDocument*) view->GetDocument();
	  document->HandleChangeImagesetName(m_txtImagesetName->GetValue());
  }
}

void PropertiesPanel::OnChangeImageFile(wxCommandEvent& WXUNUSED(event))
{
    // Show file selector.
    wxString filename = wxFileSelector(wxT("Open Image"), (const wxChar *) NULL,
        (const wxChar *) NULL, wxT("Image"),
        wxT("tga files (*.tga)|*.tga|all files (*.*)|*.*"));

    if (filename.empty())
        return;

    if (view && view->GetDocument())
    {
        // m_gridRegionProps->
        EditorDocument* document = (EditorDocument*) view->GetDocument();
        document->HandleChangeImageFile(filename);
    }
}

void PropertiesPanel::OnChangeNativeRes(wxCommandEvent& WXUNUSED(event))
{
	// Check this, because we might get this event during document closing
  if (view && view->GetDocument())
  {
	  EditorDocument* document = (EditorDocument*) view->GetDocument();
	  document->HandleChangeNativeRes(getNativeRes());
  }
}

void PropertiesPanel::OnChangeAutoScaled(wxCommandEvent& WXUNUSED(event))
{
	// Check this, because we might get this event during document closing
  if (view && view->GetDocument())
  {
	  EditorDocument* document = (EditorDocument*) view->GetDocument();
	  document->HandleChangeAutoScaled(m_ckboxAutoScaled->GetValue());
  }
}

void PropertiesPanel::OnGridAddRow(wxCommandEvent& WXUNUSED(event))
{
    // Check this, because we might get this event during document closing
    if (view && view->GetDocument())
    {
        // m_gridRegionProps->
        EditorDocument* document = (EditorDocument*) view->GetDocument();
        wxRect dim(50, 50, 10, 10);


        wxString name( "toto" );
        wxString basename( name );
        wxString count;
        while (CEGUI::WindowManager::getSingleton ().isWindowPresent (StringHelper::ToCEGUIString(name)))
        {
            count.sprintf ("%d", mUniqueGuid); 
            name = basename + count;
            mUniqueGuid++;
        }
        document->HandleAddRegion(name, dim);
    }
}

void PropertiesPanel::OnGridDelRow(wxCommandEvent& WXUNUSED(event))
{
    if( m_gridRegionProps->IsSelection())
    {
        // Check this, because we might get this event during document closing
        if (view && view->GetDocument())
        {
            wxArrayInt sel = m_gridRegionProps->GetSelectedRows();
            if (sel.Count() > 0)
            {
                wxString name = m_gridRegionProps->GetCellValue(sel.Item(0), 0);

                EditorDocument* document = (EditorDocument*) view->GetDocument();
                document->HandleDelRegion(name);
                delRegion(name);
                view->getCanvas()->delRegion(name);
            }
        }
    }
}



// we have to track the cell editiod to store the hold name
void PropertiesPanel::OnGridCellSelect(wxGridEvent& e)
{

    // Check this, because we might get this event during document closing
    if (view && view->GetDocument() && view->getCanvas() && m_bImageTrackListSel)
    {
        int row = e.GetRow();
        wxString name = m_gridRegionProps->GetCellValue(row, 0);
        view->getCanvas()->selectRegion(name);
    }
    e.Skip();
}

// we have to track the cell editiod to store the hold name
void PropertiesPanel::OnGridCellEdit(wxGridEvent& e)
{
    // Check this, because we might get this event during document closing
    if (view && view->GetDocument())
    {
        int col = e.GetCol();
        int row = e.GetRow();
        wxRect dim;
        if( col == 0)
        {
            m_oldRowName = m_gridRegionProps->GetCellValue(row, col);
        }
    }
}

void PropertiesPanel::OnGridCellChange(wxGridEvent& e)
{
    // Check this, because we might get this event during document closing
    if (view && view->GetDocument())
    {
        int row = e.GetRow();
        wxRect dim;
        wxString name = m_gridRegionProps->GetCellValue(row, 0);
        dim.x = StringHelper::WXStringToFloat(m_gridRegionProps->GetCellValue(row, 1));
        dim.y = StringHelper::WXStringToFloat(m_gridRegionProps->GetCellValue(row, 2));
        dim.width = StringHelper::WXStringToFloat(m_gridRegionProps->GetCellValue(row, 3));
        dim.height = StringHelper::WXStringToFloat(m_gridRegionProps->GetCellValue(row, 4));

        //
        EditorDocument* document = (EditorDocument*) view->GetDocument();
        int col = e.GetCol();
        if(col == 0)
        {
            if (m_oldRowName.Cmp(name) != 0)
            {
                document->HandleRenameRegion(m_oldRowName, name);
                view->getCanvas()->renameRegion(m_oldRowName, name, dim);
                m_oldRowName = name;
            }
        }

        document->HandleMoveScaleRegion(name, dim);
        view->getCanvas()->movescaleRegion(name, dim);
    }
}
