/************************************************************************
    filename:   FalSkematicPane.cpp
    created:    Sat Jul 2 2005
    author:     Paul D Turner <paul@cegui.org.uk>
*************************************************************************/
/*************************************************************************
    Crazy Eddie's GUI System (http://www.cegui.org.uk)
    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/
#include "FalSkematicPane.h"
#include "falagard/CEGUIFalWidgetLookManager.h"
#include "falagard/CEGUIFalWidgetLookFeel.h"
#include "CEGUIWindowManager.h"
#include "elements/CEGUIStaticImage.h"
#include "elements/CEGUIPushButton.h"

// Start of CEGUI namespace section
namespace CEGUI
{
    const utf8 FalagardSkematicPane::WidgetTypeName[] = "Falagard/SkematicPane";

    FalagardSkematicPane::FalagardSkematicPane(const String& type, const String& name) :
        SkematicPane(type, name)
    {
    }

    FalagardSkematicPane::~FalagardSkematicPane()
    {
    }

    void FalagardSkematicPane::populateRenderCache()
    {

        // build state name
        String stateName(isDisabled() ? "Disabled" : (isActive() ? "Active" : "Inactive"));
        const StateImagery* imagery;

        try
        {
            // get WidgetLookFeel for the assigned look.
            const WidgetLookFeel& wlf = WidgetLookManager::getSingleton().getWidgetLook(d_lookName);
            // try and get imagery for our current state
            imagery = &wlf.getStateImagery(stateName);
        }
        catch (UnknownObjectException)
        {
            // log error so we know imagery is missing, and then quit.
            return;
        }

        // peform the rendering operation.
        imagery->render(*this);
    }


    StaticImage* FalagardSkematicPane::createSketchBackgroundImage(const String& name) const
    {
        return static_cast<StaticImage*>(WindowManager::getSingleton().getWindow(getName() + "__auto_sketchimage__"));
    }

    //////////////////////////////////////////////////////////////////////////
    /*************************************************************************

        Factory Methods

    *************************************************************************/
    //////////////////////////////////////////////////////////////////////////
    Window* FalagardSkematicPaneFactory::createWindow(const String& name)
    {
        return new FalagardSkematicPane(d_type, name);
    }

    void FalagardSkematicPaneFactory::destroyWindow(Window* window)
    {
        delete window;
    }

} // End of  CEGUI namespace section
