/************************************************************************
	filename: 	CEGUISkematicPaneProperties.h
	created:	10/7/2004
	author:		Paul D Turner

	purpose:	Interface to properties for FrameWIndow class
*************************************************************************/
/*************************************************************************
    Crazy Eddie's GUI System (http://www.cegui.org.uk)
    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/
#ifndef _CEGUISkematicPaneProperties_h_
#define _CEGUISkematicPaneProperties_h_

#include "CEGUIProperty.h"


// Start of CEGUI namespace section
namespace CEGUI
{

// Start of SkematicPaneProperties namespace section
/*!
\brief
	Namespace containing all classes that make up the properties interface for the SkematicPane class
*/
namespace SkematicPaneProperties
{
/*!
\brief
	Property to access the state of the sizable setting for the SkematicPane.

	\par Usage:
		- Name: SizingEnabled
		- Format: "[text]".

	\par Where [Text] is:
		- "True" to indicate the window will be user re-sizable.
		- "False" to indicate the window will not be re-sizable by the user.
*/
class SnapToPixelEnabled : public Property
{
public:
	SnapToPixelEnabled() : Property(
		"SnapToPixelEnabled",
		"Property to get/set the state of the snap to pixel setting for the SkematicPane.  Value is either \"True\" or \"False\".",
		"True")
	{}

	String	get(const PropertyReceiver* receiver) const;
	void	set(PropertyReceiver* receiver, const String& value);
};

/*!
\brief
	Property to access the setting for whether the window frame will be displayed.

	\par Usage:
		- Name: FrameEnabled
		- Format: "[text]".

	\par Where [Text] is:
		- "True" to indicate the windows frame should be displayed.
		- "False" to indicate the windows frame should not be displayed.
*/
class SnapToGridEnabled : public Property
{
public:
	SnapToGridEnabled() : Property(
		"SnapToGridEnabled",
		"Property to get/set the state of the snap to grid setting for the SkematicPane.  Value is either \"True\" or \"False\".",
		"True")
	{}

	String	get(const PropertyReceiver* receiver) const;
	void	set(PropertyReceiver* receiver, const String& value);
};


/*!
\brief
	Property to access the setting for whether the window frame will be displayed.

	\par Usage:
		- Name: FrameEnabled
		- Format: "[text]".

	\par Where [Text] is:
		- "True" to indicate the windows frame should be displayed.
		- "False" to indicate the windows frame should not be displayed.
*/
class GridEnabled : public Property
{
public:
	GridEnabled() : Property(
		"GridEnabled",
		"Property to get/set the state of the grid setting for the SkematicPane.  Value is either \"True\" or \"False\".",
		"True")
	{}

	String	get(const PropertyReceiver* receiver) const;
	void	set(PropertyReceiver* receiver, const String& value);
};


/*!
\brief
   Property to access the N-S (up-down) sizing cursor image

   \par Usage:
      - Name: NSSizingCursorImage
      - Format: "set:<imageset> image:<imagename>".

*/
class SketchBackgroundImage : public Property
{
public:
   SketchBackgroundImage() : Property(
	   "SketchBackgroundImage",
	   "Property to get/set the sketch background image for the SkematicPane.  Value should be \"set:[imageset name] image:[image name]\".",
	   "")
   {}

   String   get(const PropertyReceiver* receiver) const;
   void   set(PropertyReceiver* receiver, const String& value);
};

/*!
\brief
   Property to access the zoom value of the sketch

   \par Usage:
      - Name: SketchZoom
		- Format: "[float]".

	\par Where [float] is the zoom factor of the sketch.

*/
class SketchZoom : public Property
{
public:
   SketchZoom() : Property(
	   "SketchZoom",
	   "Property to get/set the zoom value for the Sketch.  Value should be \"[float]\".",
	   "")
   {}

   String   get(const PropertyReceiver* receiver) const;
   void   set(PropertyReceiver* receiver, const String& value);
};

/*!
\brief
   Property to access the zoom value of the sketch

   \par Usage:
      - Name: SketchZoomMin
		- Format: "[float]".

	\par Where [float] is the minimum zoom factor of the sketch.

*/
class SketchZoomMin : public Property
{
public:
   SketchZoomMin() : Property(
	   "SketchZoomMin",
	   "Property to get/set the minimum zoom value for the Sketch.  Value should be \"[float]\".",
	   "")
   {}

   String   get(const PropertyReceiver* receiver) const;
   void   set(PropertyReceiver* receiver, const String& value);
};

/*!
\brief
   Property to access the zoom value of the sketch

   \par Usage:
      - Name: SketchZoomMax
		- Format: "[float]".

	\par Where [float] is the zoom factor of the sketch.

*/
class SketchZoomMax : public Property
{
public:
   SketchZoomMax() : Property(
	   "SketchZoomMax",
	   "Property to get/set the maximum zoom value for the Sketch.  Value should be \"[float]\".",
	   "")
   {}

   String   get(const PropertyReceiver* receiver) const;
   void   set(PropertyReceiver* receiver, const String& value);
};

/*!
\brief
   Property to access the real sketch width value of the sketch

   \par Usage:
      - Name: SketchRealWidth
		- Format: "[int]".

	\par Where [int] is the real sketch width value in pixels of the sketch.

*/
class SketchRealWidth : public Property
{
public:
   SketchRealWidth() : Property(
	   "SketchRealWidth",
	   "Property to get/set tthe real sketch width value of the sketch.  Value should be \"[int]\".",
	   "")
   {}

   String   get(const PropertyReceiver* receiver) const;
   void   set(PropertyReceiver* receiver, const String& value);
};
 
/*!
\brief
   Property to access the real sketch height value of the sketch

   \par Usage:
      - Name: SketchRealHeight
		- Format: "[int]".

	\par Where [int] is the real sketch height value in pixels of the sketch.

*/
class SketchRealHeight : public Property
{
public:
   SketchRealHeight() : Property(
	   "SketchRealHeight",
	   "Property to get/set tthe real sketch height value of the sketch.  Value should be \"[int]\".",
	   "")
   {}

   String   get(const PropertyReceiver* receiver) const;
   void   set(PropertyReceiver* receiver, const String& value);
};
 
} // End of  SkematicPaneProperties namespace section

} // End of  CEGUI namespace section


#endif	// end of guard _CEGUISkematicPaneProperties_h_
