/************************************************************************
	filename: 	CEGUISkematicPaneProperties.cpp
	created:	10/7/2004
	author:		Paul D Turner

	purpose:	Implements properties for SkematicPane class
*************************************************************************/
/*************************************************************************
    Crazy Eddie's GUI System (http://www.cegui.org.uk)
    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/
#include "elements/CEGUISkematicPaneProperties.h"
#include "elements/CEGUISkematicPane.h"
#include "CEGUIPropertyHelper.h"
#include "CEGUIExceptions.h"
#include "CEGUIFont.h"


// Start of CEGUI namespace section
namespace CEGUI
{

// Start of SkematicPaneProperties namespace section
namespace SkematicPaneProperties
{
String	SnapToPixelEnabled::get(const PropertyReceiver* receiver) const
{
	return PropertyHelper::boolToString(static_cast<const SkematicPane*>(receiver)->isSnapToPixelEnabled());
}


void	SnapToPixelEnabled::set(PropertyReceiver* receiver, const String& value)
{
	static_cast<SkematicPane*>(receiver)->setSnapToPixelEnabled(PropertyHelper::stringToBool(value));
}


String	SnapToGridEnabled::get(const PropertyReceiver* receiver) const
{
	return PropertyHelper::boolToString(static_cast<const SkematicPane*>(receiver)->isSnapToGridEnabled());
}


void	SnapToGridEnabled::set(PropertyReceiver* receiver, const String& value)
{
	static_cast<SkematicPane*>(receiver)->setSnapToGridEnabled(PropertyHelper::stringToBool(value));
}


String	GridEnabled::get(const PropertyReceiver* receiver) const
{
	return PropertyHelper::boolToString(static_cast<const SkematicPane*>(receiver)->isGridEnabled());
}


void	GridEnabled::set(PropertyReceiver* receiver, const String& value)
{
	static_cast<SkematicPane*>(receiver)->setGridEnabled(PropertyHelper::stringToBool(value));
}


String SketchBackgroundImage::get(const PropertyReceiver* receiver) const
{
    const Image* img = static_cast<const SkematicPane*>(receiver)->getSketchBackgroundImage();
    return img ? PropertyHelper::imageToString(img) : String("");
}

void SketchBackgroundImage::set(PropertyReceiver* receiver, const String &value)
{
    static_cast<SkematicPane*>(receiver)->setSketchBackgroundImage(PropertyHelper::stringToImage(value));
}



String SketchZoom::get(const PropertyReceiver* receiver) const
{
    return PropertyHelper::floatToString(static_cast<const SkematicPane*>(receiver)->getSketchZoom());
}

void SketchZoom::set(PropertyReceiver* receiver, const String &value)
{
    static_cast<SkematicPane*>(receiver)->setSketchZoom(PropertyHelper::stringToFloat(value));
}


String SketchZoomMin::get(const PropertyReceiver* receiver) const
{
    return PropertyHelper::floatToString(static_cast<const SkematicPane*>(receiver)->getSketchZoomMin());
}

void SketchZoomMin::set(PropertyReceiver* receiver, const String &value)
{
    static_cast<SkematicPane*>(receiver)->setSketchZoomMin(PropertyHelper::stringToFloat(value));
}



String SketchZoomMax::get(const PropertyReceiver* receiver) const
{
    return PropertyHelper::floatToString(static_cast<const SkematicPane*>(receiver)->getSketchZoomMax());
}

void SketchZoomMax::set(PropertyReceiver* receiver, const String &value)
{
    static_cast<SkematicPane*>(receiver)->setSketchZoomMax(PropertyHelper::stringToFloat(value));
}


String SketchRealWidth::get(const PropertyReceiver* receiver) const
{
    return PropertyHelper::intToString(static_cast<const SkematicPane*>(receiver)->getSketchRealWidth());
}

void SketchRealWidth::set(PropertyReceiver* receiver, const String &value)
{
    static_cast<SkematicPane*>(receiver)->setSketchRealWidth(PropertyHelper::stringToInt(value));
}

String SketchRealHeight::get(const PropertyReceiver* receiver) const
{
    return PropertyHelper::intToString(static_cast<const SkematicPane*>(receiver)->getSketchRealHeight());
}

void SketchRealHeight::set(PropertyReceiver* receiver, const String &value)
{
    static_cast<SkematicPane*>(receiver)->setSketchRealHeight(PropertyHelper::stringToInt(value));
}

} // End of  SkematicPaneProperties namespace section

} // End of  CEGUI namespace section
